@@ -107,7 +107,6 @@
 #'
 residency <- function(
   tz = NULL,
-  sections,
   section.order = NULL,
   datapack = NULL,
   max.interval = 60,
@@ -132,17 +131,11 @@ residency <- function(
   timestep = c("days", "hours"),
   replicates = NULL,
   GUI = c("needed", "always", "never"),
+  save.tables.locally = FALSE,
   print.releases = TRUE,
-  plot.detections.by = c("stations", "arrays"))
+  plot.detections.by = c("auto", "stations", "arrays"))
 {
 
-  if (!missing(sections))
-    warning(
-"Argument 'sections' has been deprecated and will be ignored. Please list the
-sections to which each of the arrays belong in a new 'Section' column in the
-spatial input. You can now use the argument 'section.order' to define the order
-by which sections are presented", immediate. = TRUE, call. = FALSE)
-
 # clean up any lost helpers
   deleteHelpers()
   if (file.exists(paste0(tempdir(), "/actel_debug_file.txt")))
@@ -197,7 +190,7 @@ by which sections are presented", immediate. = TRUE, call. = FALSE)
   timestep <- aux$timestep
   rm(aux)
 
-  GUI <- checkGUI(GUI)
+  GUI <- checkGUI(GUI, save.tables.locally)
 # ------------------------
 
 # Store function call
@@ -226,6 +219,7 @@ by which sections are presented", immediate. = TRUE, call. = FALSE)
     ", replicates = ", ifelse(is.null(replicates),"NULL", paste0("list(", paste(sapply(1:length(replicates), function(i) paste0("'", names(replicates)[i], "' = c('", paste(replicates[[i]], collapse = "', '"), "')")), collapse = ", "), ")")),
     ", inactive.error = ", ifelse(is.null(inactive.error), "NULL", inactive.error),
     ", GUI = '", GUI, "'",
+    ", save.tables.locally = ", ifelse(save.tables.locally, "TRUE", "FALSE"),
     ", print.releases = ", ifelse(print.releases, "TRUE", "FALSE"),
     ", plot.detections.by = '", plot.detections.by, "'",
     ")")
@@ -237,7 +231,9 @@ by which sections are presented", immediate. = TRUE, call. = FALSE)
   finished.unexpectedly <- TRUE
   on.exit({if (interactive() & finished.unexpectedly) emergencyBreak(the.function.call)}, add = TRUE)
 
-  on.exit(deleteHelpers(), add = TRUE)
+  if (!getOption("actel.debug", default = FALSE))
+    on.exit(deleteHelpers(), add = TRUE)
+
   on.exit(tryCatch(sink(), warning = function(w) {hide <- NA}), add = TRUE)
 # --------------------------------------
 
@@ -305,8 +301,8 @@ by which sections are presented", immediate. = TRUE, call. = FALSE)
 # Compile array movements
   appendTo(c("Screen", "Report"), "M: Creating movement records for the valid tags.")
   movements <- groupMovements(detections.list = detections.list, bio = bio, spatial = spatial,
-    speed.method = speed.method, max.interval = max.interval, tz = tz,
-    dist.mat = dist.mat)
+                              speed.method = speed.method, max.interval = max.interval, tz = tz,
+                              dist.mat = dist.mat)
 
   if (is.null(discard.first)) {
     aux <- names(movements)
@@ -345,8 +341,9 @@ by which sections are presented", immediate. = TRUE, call. = FALSE)
   movement.names <- names(movements)
 
   if (any(link <- !override %in% extractSignals(movement.names))) {
-    appendTo(c("Screen", "Warning", "Report"), paste0("Override has been triggered for tag ", paste(override[link], collapse = ", "), " but ",
-      ifelse(sum(link) == 1, "this tag was", "these tags were"), " not detected."))
+    appendTo(c("Screen", "Warning", "Report"), paste0("Override has been triggered for ",
+      ifelse(sum(link) == 1, "tag ", "tags "), paste(override[link], collapse = ", "), " but ",
+      ifelse(sum(link) == 1, "this signal was", "these signals were"), " not detected."))
     override <- override[!link]
   }
 
@@ -355,31 +352,32 @@ by which sections are presented", immediate. = TRUE, call. = FALSE)
     appendTo("debug", paste0("debug: Checking movement quality for tag ", tag,"."))
 
     if (is.na(match(extractSignals(tag), override))) {
-      release <- as.character(bio$Release.site[na.as.false(bio$Transmitter == tag)])
-      release <- unlist(strsplit(with(spatial, release.sites[release.sites$Standard.name == release, "Array"]), "|", fixed = TRUE))
+      output <- checkMinimumN(movements = movements[[tag]], tag = tag, minimum.detections = minimum.detections)
 
-      output <- checkMinimumN(movements = movements[[i]], tag = tag, minimum.detections = minimum.detections)
+      output <- checkImpassables(movements = output, tag = tag, bio = bio, detections = detections.list[[tag]], 
+                                 spatial = spatial, dotmat = dotmat, GUI = GUI, save.tables.locally = save.tables.locally)
 
-      output <- checkImpassables(movements = output, tag = tag, dotmat = dotmat, GUI = GUI)
-
-      output <- checkJumpDistance(movements = output, release = release, tag = tag, dotmat = dotmat,
-                                  jump.warning = jump.warning, jump.error = jump.error, GUI = GUI)
+      output <- checkJumpDistance(movements = output, bio = bio, tag = tag, dotmat = dotmat, paths = paths, arrays = arrays,
+                                  spatial = spatial, jump.warning = jump.warning, jump.error = jump.error, GUI = GUI,
+                                  detections = detections.list[[tag]], save.tables.locally = save.tables.locally)
 
       if (do.checkSpeeds) {
         temp.valid.movements <- simplifyMovements(movements = output, tag = tag, bio = bio, discard.first = discard.first,
-          speed.method = speed.method, dist.mat = dist.mat)
-        output <- checkSpeeds(movements = output, tag = tag, valid.movements = temp.valid.movements,
-          speed.warning = speed.warning, speed.error = speed.error, GUI = GUI)
+                                                  speed.method = speed.method, dist.mat = dist.mat)
+        output <- checkSpeeds(movements = output, tag = tag, valid.movements = temp.valid.movements, 
+                              detections = detections.list[[tag]], speed.warning = speed.warning, 
+                              speed.error = speed.error, GUI = GUI, save.tables.locally = save.tables.locally)
         rm(temp.valid.movements)
       }
 
       if (do.checkInactiveness) {
-        output <- checkInactiveness(movements = output, tag = tag, detections.list = detections.list[[tag]],
-          inactive.warning = inactive.warning, inactive.error = inactive.error,
-          dist.mat = dist.mat, GUI = GUI)
+        output <- checkInactiveness(movements = output, tag = tag, detections = detections.list[[tag]],
+                                    inactive.warning = inactive.warning, inactive.error = inactive.error,
+                                    dist.mat = dist.mat, GUI = GUI, save.tables.locally = save.tables.locally)
       }
     } else {
-      output <- overrideValidityChecks(moves = movements[[i]], tag = names(movements)[i], GUI = GUI) # nocov
+      output <- overrideValidityChecks(moves = movements[[tag]], tag = tag, detections = detections.list[[tag]], # nocov
+                                       GUI = GUI, save.tables.locally = save.tables.locally) # nocov
     }
     return(output)
   })
@@ -404,23 +402,16 @@ by which sections are presented", immediate. = TRUE, call. = FALSE)
   movements <- updateValidity(arrmoves = movements, secmoves = section.movements)
 
   # compile valid movements
-  appendTo(c("Screen", "Report"), "M: Filtering valid array and section movements.")
+  appendTo(c("Screen", "Report"), "M: Filtering valid array movements.")
 
-  valid.movements <- lapply(seq_along(movements), function(i){
-    output <- simplifyMovements(movements = movements[[i]], tag = names(movements)[i], bio = bio, discard.first = discard.first,
-      speed.method = speed.method, dist.mat = dist.mat)
-  })
-  names(valid.movements) <- names(movements)
-  valid.movements <- valid.movements[!unlist(lapply(valid.movements, is.null))]
+  valid.movements <- assembleValidMoves(movements = movements, bio = bio, discard.first = discard.first,
+                                         speed.method = speed.method, dist.mat = dist.mat)
 
-  section.movements <- lapply(seq_along(valid.movements), function(i) {
-    tag <- names(valid.movements)[i]
-    appendTo("debug", paste0("debug: Compiling valid section movements for tag ", tag,"."))
-    output <- sectionMovements(movements = valid.movements[[i]], spatial = spatial, valid.dist = attributes(dist.mat)$valid)
-    return(output)
-  })
-  names(section.movements) <- names(valid.movements)
+  appendTo(c("Screen", "Report"), "M: Filtering valid section movements.")
 
+  section.movements <- assembleValidSecMoves(valid.moves = valid.movements, spatial = spatial, 
+                                             valid.dist = attributes(dist.mat)$valid)
+  
   # Grab summary information
   appendTo(c("Screen", "Report"), "M: Compiling residency objects.")
 
@@ -430,20 +421,22 @@ by which sections are presented", immediate. = TRUE, call. = FALSE)
 
   last.seen <- as.data.frame.matrix(with(status.df, table(Group, Status)))
 
+  residency.list <- getResidency(movements = section.movements, spatial = spatial)
+
+  # Circular data
   aux <- list(valid.movements = valid.movements, section.movements = section.movements,
-    spatial = spatial, rsp.info = list(bio = bio, analysis.type = "residency"))
+              spatial = spatial, rsp.info = list(bio = bio, analysis.type = "residency"))
   array.times <- getTimes(input = aux, move.type = "array", event.type = "arrival", n.events = "all")
 
-  section.times <- list(
-    arrival = getTimes(input = aux, move.type = "section", event.type = "arrival", n.events = "all"),
-    departure = getTimes(input = aux, move.type = "section", event.type = "departure", n.events = "all"))
+  section.times <- list(arrival = getTimes(input = aux, move.type = "section", 
+                                           event.type = "arrival", n.events = "all"),
+                        departure = getTimes(input = aux, move.type = "section", 
+                                             event.type = "departure", n.events = "all"))
   rm(aux)
 
-  residency.list <- getResidency(movements = section.movements, spatial = spatial)
-
   if (length(residency.list) == 0) {
     emergencyBreak()
-    stop("No tags have enough data for residency analysis. Consider running explore() instead.\n", call. = FALSE)
+    stopAndReport("No tags have enough data for residency analysis. Consider running explore() instead.\n")
   }
 
   if (timestep == "days")
@@ -469,7 +462,7 @@ by which sections are presented", immediate. = TRUE, call. = FALSE)
   })
   names(group.ratios) <- unique(bio$Group)
 
-  appendTo("Screen", "M: Validating detections...")
+  appendTo("Screen", "M: Validating detections.")
 
   recipient <- validateDetections(detections.list = detections.list, movements = valid.movements)
   detections <- recipient$detections
@@ -500,7 +493,6 @@ by which sections are presented", immediate. = TRUE, call. = FALSE)
 
 # ----------
 
-
 # ---------------
 
 # wrap up in-R objects
@@ -509,7 +501,8 @@ by which sections are presented", immediate. = TRUE, call. = FALSE)
   efficiency <- efficiency[1:3]
 
   # extra info for potential RSP analysis
-  rsp.info <- list(analysis.type = "residency", analysis.time = the.time, bio = bio, tz = tz, actel.version = utils::packageVersion("actel"))
+  rsp.info <- list(analysis.type = "residency", analysis.time = the.time, bio = bio, 
+                   tz = tz, actel.version = utils::packageVersion("actel"))
 
   if (!is.null(override))
     override.fragment <- paste0('<span style="color:red">Manual mode has been triggered for **', length(override),'** tag(s).</span>\n')
@@ -773,8 +766,8 @@ printResidencyRmd <- function(
 
   work.path <- paste0(tempdir(), "/actel_report_auxiliary_files/")
 
-  if (any(grepl("Unknown", spatial$stations$Standard.name))) {
-    unknown.fragment <- paste0('<span style="color:red"> Number of relevant unknown receivers: **', sum(grepl("Unknown", spatial$stations$Standard.name)), '**</span>\n')
+ if (!is.null(spatial$unknowns)) {
+    unknown.fragment <- paste0('<span style="color:red"> Number of relevant unknown receivers: **', sum(sapply(spatial$unknowns, length)), '** (of which ', length(spatial$unknowns$included),' were included)</span>\n')
   } else {
     unknown.fragment <- ""
   }
@@ -782,7 +775,7 @@ printResidencyRmd <- function(
     sensor.fragment <- paste0("### Sensor plots
 
 Note:
-  : You can choose to paint the values by section by setting `plot.detections.by = 'arrays'` during the analysis.
+  : The colouring in these plots will follow that of the individual detection plots, which can be modified using `plot.detections.by`.
   : The data used for these graphics is stored in the `valid.detections` object.
   : You can replicate these graphics and edit them as needed using the `plotSensors()` function.
 
@@ -792,6 +785,21 @@ Note:
   }
 
   report <- readr::read_file(paste0(tempdir(), "/temp_log.txt"))
+  report <- gsub("(\\\\|\")", "\\\\\\1", report)
+
+  if (file.exists(paste0(tempdir(), '/temp_warnings.txt'))) {
+    warning.messages <- gsub("\\r", "", readr::read_file(paste0(tempdir(), '/temp_warnings.txt')))
+    warning.messages <- gsub("(\\\\|\")", "\\\\\\1", warning.messages)
+  } else {
+    warning.messages <- 'No warnings were raised during the analysis.'
+  }
+
+  if (file.exists(paste0(tempdir(), '/temp_comments.txt'))) {
+    comment.fragment <- gsub("\\r", "", readr::read_file(paste0(tempdir(), '/temp_comments.txt')))
+    comment.fragment <- gsub("(\\\\|\")", "\\\\\\1", comment.fragment)
+  } else {
+    comment.fragment <- 'No comments were included during the analysis.'
+  }
 
   oldoptions <- options(knitr.kable.NA = "-")
   on.exit(options(oldoptions), add = TRUE)
@@ -854,17 +862,13 @@ Arrays with the same background belong to the same section. Release sites are ma
 ### Warning messages
 
 ```{r warnings, echo = FALSE, comment = NA}
-cat("', ifelse(file.exists(paste0(tempdir(), '/temp_warnings.txt')),
-  gsub("\\r", "", readr::read_file(paste0(tempdir(), '/temp_warnings.txt'))),
-  'No warnings were raised during the analysis.'), '")
+cat("', warning.messages, '")
 ```
 
 ### User comments
 
 ```{r comments, echo = FALSE, comment = NA}
-cat("', ifelse(file.exists(paste0(tempdir(), '/temp_comments.txt')),
-  gsub("\\r", "", readr::read_file(paste0(tempdir(), '/temp_comments.txt'))),
-  'No comments were included during the analysis.'), '")
+cat("', comment.fragment, '")
 ```
 
 ', ifelse(biometric.fragment == '', '', paste0('### Biometric graphics
@@ -995,7 +999,7 @@ Note:
 ### Full log
 
 ```{r log, echo = FALSE, comment = NA}
-cat("', gsub("\\r", "", readr::read_file(paste0(tempdir(), '/temp_log.txt'))), '")
+cat("', gsub("\\r", "", report), '")
 ```
 
 '), fill = TRUE)
