@@ -21,7 +21,7 @@ if (any(missing.packages)) {
       	"' to operate. Please install ", ifelse(sum(missing.packages) > 1, "them", "it"), " before proceeding.\n"), fixed = TRUE)
   })
 } else {
-	if (suppressWarnings(require("rgdal"))) {
+	if (suppressWarnings(require("rgdal")) & suppressWarnings(require("gdistance"))) {
 		tests.home <- getwd()
 		setwd(tempdir())
 
@@ -98,11 +98,11 @@ if (any(missing.packages)) {
 			coord.x = "x.32632", coord.y = "y.32632")
 		t.layer <- transitionLayer(base.raster)
 
-		test_that("distancesMatrix produces a warning when there are viable passages between stations", {
+		test_that("distancesMatrix produces a warning when there are no viable passages between stations", {
 			expect_warning(dist.mat <- distancesMatrix(t.layer = t.layer,
 		  		coord.x = "x.32632", coord.y = "y.32632", actel = TRUE),
 			"At least one station is completely blocked off from the remaining stations by land. Filling
-the respective fields with NA. If your fish was expected to travel around the areas present
+the respective fields with NA. If your animals were expected to travel around the areas present
 in the shape file, consider applying a 'buffer' when calculating the transition layer. This
 will artificially add water space around the shape file.", fixed = TRUE)
 		})
@@ -178,6 +178,15 @@ will artificially add water space around the shape file.", fixed = TRUE)
 			"Extending the shape ranges with open water to ensure the stations fit inside it.", fixed = TRUE)
 		})
 
+		xspatial <- example.spatial[8:11, ]
+		xspatial$x.32632 <- c(453500, 453400, 452047, 452975)
+		xspatial$y.32632 <- c(6242800, 6242630, 6242387, 6241169)
+		test_that("loadShape expands the grid range if the spatial objects are outside the shape range.", {
+			expect_warning(loadShape(path = tests.home, shape = "aux_transitionLayer.shp", size = 10,
+					coord.x = "x.32632", coord.y = "y.32632", spatial = xspatial),
+			"Station 'Station 7' is not placed in water! This can cause several problems.", fixed = TRUE)
+		})
+
 		file.remove("spatial.csv")
 		setwd(tests.home)
 		rm(list = ls())
