@@ -1,3 +1,5 @@
+# keep in mind that tests with skip_on_cran must be tested line by line.
+
 oldtz <- Sys.getenv('TZ', unset = NA)
 Sys.setenv(TZ = 'UTC')
 
@@ -154,8 +156,8 @@ test_that("migration can handle multiple expected first arrays", {
 	xspatial <- example.spatial
 	xspatial$Array[18] <- "A1|A2"
 	write.csv(xspatial, "spatial.csv", row.names = FALSE)
-	expect_message(suppressWarnings(output <- migration(sections = c("A", "Fjord", "Sea"),
-		tz = 'Europe/Copenhagen', report = TRUE, success.arrays = "A9", GUI = "never")),
+	expect_message(suppressWarnings(output <- migration(tz = 'Europe/Copenhagen', 
+		report = TRUE, success.arrays = "A9", GUI = "never")),
 		"Multiple possible first arrays detected for release site 'RS1'.", fixed = TRUE)
 })
 
